import os
import shutil
import sys
import tempfile
from subprocess import PIPE, Popen, TimeoutExpired

from dotenv import load_dotenv

load_dotenv()


def decodeResults(results):
    return (res.strip().decode("utf-8") for res in results)


def refineCode(code):
    if "```" in code:
        code = code.split("```")[1]
        if code.startswith("python"):
            code = code[len("python") :].strip()
    return code


def runPython(code, timeout_sec=10):
    code = refineCode(code)
    with tempfile.NamedTemporaryFile("w") as tmp:
        tmp.write(code)
        tmp.flush()
        environmental_variables = {"OPENAI_API_KEY": os.getenv("OPENAI_API_KEY")}
        python_path = shutil.which("python")
        if not python_path:  # shows 'which' returns None
            python_path = sys.executable
        process = Popen(
            [python_path, tmp.name],
            env=environmental_variables,
            stdout=PIPE,
            stderr=PIPE,
        )
        try:
            output, err = decodeResults(process.communicate(timeout=timeout_sec))
            success = len(err) == 0
        except TimeoutExpired:
            process.kill()
            success = True
            output = err = ""
        return output, err, success


def runStreamlit(code, openai_api_key):
    """
    Runs the provided code as a Streamlit application and returns the process ID.

    Args:
        code (str): The code of the Streamlit application.

    Returns:
        int: The process ID of the Streamlit application.
    """
    with tempfile.NamedTemporaryFile("w", suffix=".py", delete=False) as tmp:
        tmp.write(code)
        tmp.flush()
        environmental_variables = {
            "OPENAI_API_KEY": openai_api_key,
            "STREAMLIT_SERVER_PORT": "8502",
        }
        streamlit_path = shutil.which("streamlit")
        process = Popen([streamlit_path, "run", tmp.name], env=environmental_variables)
        return process.pid



from dotenv import load_dotenv
from langchain.chat_models import ChatOpenAI
from langchain.schema import AIMessage, HumanMessage, SystemMessage
from tqdm import tqdm

load_dotenv()

llm = ChatOpenAI(model="gpt-3.5-turbo-16k", temperature=0)


def refine_code(code):
    if "```" in code:
        code = code.split("```")[1]
        if code.startswith("python"):
            code = code[len("python") :].strip()
    return code


def getCode(doc):
    prompt = f"""
    Transfer the below document to a valid python code by preserving the standard texts as comments

    {doc}
    """
    messages = [HumanMessage(content=prompt)]
    code = llm(messages).content
    return refine_code(code)


def getGoals(doc):
    prompt = f"""
    For which project ideas, the techniques inside the below code could be used?
    Create a list of project idea

    {doc}
    """
    messages = [HumanMessage(content=prompt)]
    code = llm(messages).content
    return refine_code(code)


import os

ROOT = "docs"

# Fill codes folder
for filename in tqdm(os.listdir(ROOT)):
    filepath = os.path.join(ROOT, filename)
    out_filename = filename.replace(".txt", ".py")
    out_path = f"examples/codes/{out_filename}"
    if os.path.exists(out_path):
        continue
    with open(filepath) as f:
        doc = f.read()
    code = getCode(doc)
    with open(out_path, "w") as f:
        f.write(code)

CODE_ROOT = "examples/codes/"
# Fill goals folder
for filename in tqdm(os.listdir(CODE_ROOT)):
    filepath = os.path.join(CODE_ROOT, filename)
    out_filename = filename.replace(".py", ".md")
    out_path = f"examples/goals/{out_filename}"
    if os.path.exists(out_path):
        continue
    with open(filepath) as f:
        code = f.read()
    goals = getGoals(code)
    with open(out_path, "w") as f:
        f.write(goals)



import logging
import os

import fire
import langchain
from dotenv import load_dotenv
from langchain.chains import ConversationalRetrievalChain
from langchain.chat_models import ChatOpenAI
from langchain.document_loaders import TextLoader
from langchain.embeddings import HuggingFaceEmbeddings
from langchain.schema import HumanMessage
from langchain.text_splitter import (CharacterTextSplitter, Language,
                                     RecursiveCharacterTextSplitter)
from langchain.vectorstores import Chroma
from tqdm import tqdm

load_dotenv()


class LangChainExpert:
    def __init__(
        self,
        model_name="gpt-3.5-turbo-16k",
        persist_directory="langchain_db_python",
        device="cuda",
        distance_metric="cos",
        fetch_k=20,
        maximal_marginal_relevance=True,
        k=20,
    ):
        self.root_dir = "/".join(langchain.__file__.split("/")[:-1])
        self.model_name = model_name
        self.persist_directory = persist_directory
        self.device = device
        self.distance_metric = distance_metric
        self.fetch_k = fetch_k
        self.maximal_marginal_relevance = maximal_marginal_relevance
        self.k = k
        self.embeddings = HuggingFaceEmbeddings(model_kwargs={"device": device})
        self.__initModels()

    def __constructDB(self, split_type="python"):
        if os.path.exists(self.persist_directory):
            logging.info("DB has been found and fetched")
            self.db = Chroma(
                persist_directory=self.persist_directory,
                embedding_function=self.embeddings,
            )
        else:
            logging.info("DB not found, creation started...")
            docs = []
            for dirpath, _, filenames in tqdm(os.walk(self.root_dir)):
                for file in filenames:
                    if file.endswith(".py") and "/.venv/" not in dirpath:
                        try:
                            loader = TextLoader(
                                os.path.join(dirpath, file), encoding="utf-8"
                            )
                            docs.extend(loader.load_and_split())
                        except Exception as e:
                            pass
            if split_type == "python":
                python_splitter = RecursiveCharacterTextSplitter.from_language(
                    language=Language.PYTHON, chunk_size=1000, chunk_overlap=0
                )
                texts = python_splitter.split_documents(docs)
            else:
                text_splitter = CharacterTextSplitter(chunk_size=1000, chunk_overlap=0)
                texts = text_splitter.split_documents(docs)
            self.db = Chroma.from_documents(
                texts, self.embeddings, persist_directory=self.persist_directory
            )

    def __constructRetriever(self):
        self.__constructDB()
        self.retriever = self.db.as_retriever()
        self.retriever.search_kwargs["distance_metric"] = self.distance_metric
        self.retriever.search_kwargs["fetch_k"] = self.fetch_k
        self.retriever.search_kwargs[
            "maximal_marginal_relevance"
        ] = self.maximal_marginal_relevance
        self.retriever.search_kwargs["k"] = self.k
        logging.info("Retriever has been succesfully constructed")

    def __initModels(self):
        self.__constructRetriever()
        self.model = ChatOpenAI(model_name=self.model_name, temperature=0)
        self.qa = ConversationalRetrievalChain.from_llm(
            self.model, retriever=self.retriever
        )
        self.qa_chat_history = []
        self.chat_history = []

    def askToModel(self, question, chat_history=False):
        messages = [HumanMessage(content=question)]
        if chat_history:
            messages = self.chat_history + messages
        return self.model(messages).content

    def ask(self, question, add_history=False):
        result = self.qa({"question": question, "chat_history": self.qa_chat_history})
        if add_history:
            self.chat_history.append((question, result["answer"]))
        return result["answer"]

    def debug(self, error):
        question = f"""
            How to solve that error below?

            {error}
            """
        return self.ask(question)


if __name__ == "__main__":
    expert = LangChainExpert()
    fire.Fire(expert.ask)



from setuptools import setup, find_packages

setup(
    name="demogpt",
    version="1.1.1.2",
    url="https://github.com/melih-unsal/DemoGPT",
    author="Melih Unsal",
    author_email="melih@demogpt.io",
    description="Description of my package",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "streamlit",
        "langchain",
        "openai",
        "python-dotenv"
    ],
    entry_points={
        "console_scripts": [
            "demogpt = prompt_based.app:main",
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
